unit AnalogueClock;

{ Duncan Parsons 2002
This Component is freeware, but I am interested in where it end up!!
Drop me a line on duncanparsons@hotmail.com

This is a Static Analogue Clock face. There are a lot of moving clock faces around,
but I needed one which didn't (for inputting purposes).

If you make any good changes, let me know!
Happy Coding
Duncan Parsons}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TAnalogueHandShowing=(ahsHour,ahsMinute,ahsSecond);
  TAnalogueHandsShowing=set of TAnalogueHandShowing;

  TAnalogueHand=class(TPersistent)
    fColour:TColor;
    fBackLength:integer;
    fThickness:integer;
  published
    property Colour:TColor read fColour write fColour;
    property BackLength:integer read fBackLength write fBackLength;
    property Thickness:integer read fThickness write fThickness;
  end;

  TMarkerShape=(msCircle,msSquare);
  TAnalogueMarker=class(TPersistent)
    fShape:TMarkerShape;
    fBorderColour:TColor;
    fInnerColour:TColor;
    fInnerColourPM:TColor;
    fUsePMColour:boolean;
    fSize:integer;
    fPenWidth:integer;
  published
    property Shape:TMarkerShape read fShape write fShape;
    property BorderColour:TColor read fBorderColour write fBorderColour;
    property InnerColour:TColor read fInnerColour write fInnerColour;
    property InnerColourPM:TColor read fInnerColourPM write fInnerColourPM;
    property UsePMColour:Boolean read fUsePMColour write fUsePMColour;
    property Size:Integer read fSize write fSize;
    property PenWidth:integer read fPenWidth write fPenWidth;
  end;

  TAnalogueBevel=(abNone,abRaised,abSunken);

  TAnalogueClock = class(TGraphicControl)
  private
    { Private declarations }
    fHour,fMinute,fSec:word;
    XCentre, YCentre, Radius: Integer;
    fHourHand,fMinuteHand,FSecHand:TAnalogueHand;
    fMarkerColour,fMarkerOutline:TColor;
    fHandsShowing:TAnalogueHandsShowing;
    fMarkers:TAnalogueMarker;
    fTransparent:boolean;
    fBevel:TAnalogueBevel;
  protected
    { Protected declarations }
    procedure SetBevel(value:TAnalogueBevel);
    procedure SetHand(index:integer;value:TAnalogueHand);
    function  GetTime:TDateTime;
    procedure SetTime(Value:TDateTime);
    procedure SetTimePart(index:integer;Value:word);
    procedure SetMarkers(Value:TAnalogueMarker);
    procedure SetHandsShowing(Value:TAnalogueHandsShowing);
    procedure SetTransparent(Value:Boolean);
    procedure DrawHand (XCentre, YCentre,Radius, BackRadius: Integer;
                                 Angle: Real);
    procedure Paint;override;
  public
    { Public declarations }
    property TimeShown:TDateTime              read GetTime     write SetTime;
    constructor create(AOwner:TComponent);override;
    destructor  destroy;override;
  published
    { Published declarations }
    property Align;
    property Hint;
    property ShowHint;
    property ParentShowHint;
    property Color;
    property ParentColor;
    property Visible;
    property Bevel:TAnalogueBevel             read fBevel write SetBevel default abRaised;
    property HourHand:TAnalogueHand   index 0 read fHourHand   write SetHand;
    property MinuteHand:TAnalogueHand index 1 read fMinuteHand write SetHand;
    property SecHand:TAnalogueHand    index 2 read fSecHand    write SetHand;
    property Hour:word                index 0 read fHour       write SetTimePart;
    property Minute:word              index 1 read fMinute     write SetTimePart;
    property Second:word              index 2 read fSec        write SetTimePart;
    property HourMarkers:TAnalogueMarker read fMarkers write SetMarkers;
    property HandsShowing:TAnalogueHandsShowing read fHandsShowing write SetHandsShowing;
    property Transparent:boolean      read fTransparent write SetTransparent default true;
  end;

procedure Register;

implementation


procedure Register;
begin
  RegisterComponents('Tools', [TAnalogueClock]);
end;

constructor TAnalogueClock.create(AOwner:TComponent);
var msec:word;
begin
     inherited create(AOwner);
     //Set the size
     width:=70;
     height:=70;
     fTransparent:=True;
     fBevel:=abRaised;
     //Set up the Hour Markers
     fMarkers:=TAnalogueMarker.create;
     with fMarkers do
          begin
               Shape:=msCircle;
               BorderColour:=clBlue;
               InnerColour:=clWhite;
               InnerColourPM:=clYellow;
               UsePMColour:=True;
               Size:=1;
               PenWidth:=1;
          end;
     //Set up the Hand Objects
     fHourHand:=TAnalogueHand.create;
     with fHourHand do
          begin
               colour:=clBlue;
               BackLength:=0;
               Thickness:=2;
          end;
     fMinuteHand:=TAnalogueHand.create;
     with fMinuteHand do
          begin
               colour:=clRed;
               BackLength:=0;
               Thickness:=2;
          end;
     fSecHand:=TAnalogueHand.create;
     with fSecHand do
          begin
               colour:=clRed;
               BackLength:=-1;
               Thickness:=1;
          end;
     //Set to the current Time
     DecodeTime(Time,fHour,fMinute,fSec,msec);
     //Set some defaults
     fMarkerColour:=clWhite;
     fMarkerOutline:=clBlue;
     fHandsShowing:=[ahsHour..ahsSecond];
end;

destructor  TAnalogueClock.destroy;
begin
     fSecHand.free;
     fMinuteHand.free;
     fHourHand.free;
     fMarkers.free;
     inherited;
end;

procedure TAnalogueClock.SetBevel(value:TAnalogueBevel);
begin
     if fBevel<>Value then
        begin
             fBevel:=Value;
             invalidate;
        end;
end;

procedure TAnalogueClock.SetHand(index:integer;value:TAnalogueHand);
begin
     case index of
          0 : begin
                   if fHourHand<>Value then
                      begin
                           fHourHand:=Value;
                      end;
              end;
          1 : begin
                   if fMinuteHand<>Value then
                      begin
                           fMinuteHand:=Value;
                      end;
              end;
          2 : begin
                   if fSecHand<>Value then
                      begin
                           fSecHand:=Value;
                      end;
              end;
     end;
     invalidate;
end;

function  TAnalogueClock.GetTime:TDateTime;
begin
     result:=EncodeTime(fHour,fMinute,fSec,0);
end;

procedure TAnalogueClock.SetTime(Value:TDateTime);
var msec:word;
begin
     DecodeTime(Value,fHour,fMinute,fSec,msec);
     invalidate;
end;

procedure TAnalogueClock.SetTimePart(index:integer;Value:word);
begin
     case index of
          0 : begin
                   if Value<0 then Value:=0;
                   if Value>23 then value:=23;
                   if fHour<>value then
                      begin
                           fHour:=value;
                           invalidate;
                      end;
              end;
          1 : begin
                   if Value<0 then Value:=0;
                   if Value>59 then value:=59;
                   if fMinute<>value then
                      begin
                           fMinute:=value;
                           invalidate;
                      end;
              end;
          2 : begin
                   if Value<0 then Value:=0;
                   if Value>59 then value:=59;
                   if fSec<>value then
                      begin
                           fSec:=value;
                           invalidate;
                      end;
              end;
     end;
end;

procedure TAnalogueClock.SetMarkers(Value:TAnalogueMarker);
begin
     fMarkers:=Value;
     invalidate;
end;

procedure TAnalogueClock.SetHandsShowing(Value:TAnalogueHandsShowing);
begin
     if fHandsShowing<>Value then
        begin
             fHandsShowing:=value;
             invalidate;
        end;
end;

procedure TAnalogueClock.SetTransparent(Value:Boolean);
begin
     if fTransparent<>Value then
        begin
             fTransparent:=Value;
             invalidate;
        end;
end;

procedure TAnalogueClock.DrawHand (XCentre, YCentre,
  Radius, BackRadius: Integer; Angle: Real);
begin
     Canvas.MoveTo (XCentre - Round (BackRadius * Cos (Angle)),
                      YCentre - Round (BackRadius * Sin (Angle)));
     Canvas.LineTo (XCentre + Round (Radius * Cos (Angle)),
                      YCentre + Round (Radius * Sin (Angle)));
end;

procedure TAnalogueClock.Paint;
var Angle: Real;
    I, X, Y, Size,tmp: Integer;
    R:TRect;
begin
     if not(fTransparent) then
        begin
             Canvas.Brush.Color:=Color;
             Canvas.FillRect(ClientRect);
        end;

     {compute the middle}
     XCentre := ClientWidth div 2;
     YCentre := ClientHeight div 2;
     if XCentre > YCentre then
        Radius := YCentre - 8
       else
        Radius := XCentre - 8;

     Canvas.Pen.Color := fMarkers.BorderColour;
     Canvas.Pen.Mode := pmCopy;
     if (fMarkers.UsePMColour) and (fHour>11) then
        Canvas.Brush.Color := fMarkers.InnerColourPM
       else
        Canvas.Brush.Color := fMarkers.InnerColour;
     Canvas.Pen.Width:=fMarkers.Penwidth;//1;
     Size := (fMarkers.Size*Radius) div 25 +1;//25{50} + 1;

     //Draw Clock
     for I := 0 to 11 do
         begin
              Angle := 2 * Pi * (I + 9) / 12;
              X := XCentre - Round (Radius * Cos (Angle));
              Y := YCentre - Round (Radius * Sin (Angle));
              //Ensure it will all fit...
              if (X+Size+fMarkers.PenWidth)>width then X:=Width-Size-fMarkers.PenWidth;
              if (X-Size-fMarkers.PenWidth)<0 then X:=Size+fMarkers.PenWidth;
              if (Y+Size+fMarkers.PenWidth)>height then Y:=Height-Size-fMarkers.PenWidth;
              if (Y-Size-fMarkers.PenWidth)<0 then Y:=Size+fMarkers.PenWidth;
              case fMarkers.Shape of
                   msCircle: Canvas.Ellipse (X - Size, Y - Size, X + Size, Y + Size);
                   msSquare: Canvas.Rectangle(X - Size, Y - Size, X + Size, Y + Size);
              end;
         end;
     if ahsMinute in fHandsShowing then
        begin
             //Draw Minute Hand
             Canvas.Pen.Width := fMinuteHand.Thickness;
             Canvas.Pen.Color := fMinuteHand.Colour;
             Angle := 2 * Pi * (fMinute+45) / 60;
             DrawHand (XCentre, YCentre,
                      Radius * 90 div 100, fMinuteHand.BackLength, Angle);
        end;
     if ahsHour in fHandsShowing then
        begin
             //Draw Hour Hand
             Canvas.Pen.Width := fHourHand.Thickness;
             Canvas.Pen.Color := fHourHand.Colour;
             Angle := 2 * Pi * (fHour + 9 + fMinute / 60) / 12;
             DrawHand (XCentre, YCentre,
                      Radius * 70 div 100, fHourHand.BackLength, Angle);
        end;
     if ahsSecond in fHandsShowing then
        begin
             //Draw Second Hand
             Canvas.Pen.Width := fSecHand.Thickness;
             Canvas.Pen.Color := fSecHand.Colour;
             Canvas.Pen.Mode := pmNotXor;
             Angle := 2 * Pi * (Second+45) / 60;
             //Get BackLength
             if fSecHand.BackLength<0 then
                tmp:=Radius * 30 div 100
               else
                tmp:=fSecHand.BackLength;
             DrawHand (XCentre, YCentre, Radius,tmp, Angle);
        end;

     R:=ClientRect;
     case fBevel of
          abRaised : DrawEdge(Canvas.Handle,R,BDR_RAISEDINNER,BF_RECT);
          abSunken : DrawEdge(Canvas.Handle,R,BDR_SUNKENOUTER,BF_RECT);
     end;
end;

end.
